@@ -61,7 +61,7 @@ let is_diff_index file =
 let is_pdiff file =
   Filename.basename file <> "Index" && diff_index_dir file
 
-let immutable_suffixes = [".deb"; ".dsc"; ".tar.gz"; ".diff.gz"]
+let immutable_suffixes = [".deb"; ".dsc"; ".tar.gz"; ".diff.gz"; ".udeb"]
 
 let immutable file =
   List.exists (Filename.check_suffix file) immutable_suffixes || is_pdiff file
